#ifndef BUILD_CONFIGURATION_GROUP
#define BUILD_CONFIGURATION_GROUP

/*****************************************************************************\
*                                                                             *
*  Name   : build configuration                                               *
*  Author : Chris Koeritz                                                     *
*                                                                             *
*******************************************************************************
* Copyright (c) 1995-$now By Author.  This program is free software; you can  *
* redistribute it and/or modify it under the terms of the GNU General Public  *
* License as published by the Free Software Foundation; either version 2 of   *
* the License or (at your option) any later version.  This is online at:      *
*     http://www.fsf.org/copyleft/gpl.html                                    *
* Please send any updates to: fred@gruntose.com                               *
\*****************************************************************************/

/*! @file build_configuration.h
  @brief Contains definitions that control how libraries and programs are built.

  These definitions can be used for declaring classes and functions that need
  to be called from a DLL or that need to be compiled into a DLL.  The import
  style is used to declare that the item is being pulled out of a DLL.  The
  export style is used to declare that the item is being included in a DLL for
  use by other functions.
*/

#include "definitions.h"

// Here is an example of a dll header to be used by a particular library
// (presumably the gurpta library):
//
// #ifndef GURPTA_DLL_HEADER
// #define GURPTA_DLL_HEADER
//   // define BUILD_GURPTA when you are creating the dll and
//   // define USE_HOOPLE_DLLS when you are importing code from the dll.
//   #include <basis/build_configuration.h>
//   #if defined(BUILD_GURPTA)
//     #define GURPTA_CLASS_STYLE    HOOPLE_DLL_EXPORT_CLASS
//     #define GURPTA_FUNCTION_STYLE HOOPLE_DLL_EXPORT_FUNCTION
//   #elif defined(USE_HOOPLE_DLLS)
//     #define GURPTA_CLASS_STYLE    HOOPLE_DLL_IMPORT_CLASS
//     #define GURPTA_FUNCTION_STYLE HOOPLE_DLL_IMPORT_FUNCTION
//   #else
//     #define GURPTA_CLASS_STYLE
//     #define GURPTA_FUNCTION_STYLE
//   #endif
// #endif // outer guard.

#ifdef __WIN32__
  // Win 32 Exports
  #define HOOPLE_DLL_EXPORT_CLASS     __declspec(dllexport)
  #define HOOPLE_DLL_EXPORT_FUNCTION  __declspec(dllexport)
  // Win 32 Imports
  #define HOOPLE_DLL_IMPORT_CLASS     __declspec(dllimport)
  #define HOOPLE_DLL_IMPORT_FUNCTION  __declspec(dllimport)
#else
  // no known requirements for these tags, so set them to nothing.
  #define HOOPLE_DLL_EXPORT_CLASS
  #define HOOPLE_DLL_EXPORT_FUNCTION
  #define HOOPLE_DLL_IMPORT_CLASS
  #define HOOPLE_DLL_IMPORT_FUNCTION
#endif

// legacy macros.
#define INOVA_DLL_EXPORT_CLASS HOOPLE_DLL_EXPORT_CLASS
#define INOVA_DLL_EXPORT_FUNCTION HOOPLE_DLL_EXPORT_FUNCTION
#define INOVA_DLL_IMPORT_CLASS HOOPLE_DLL_IMPORT_CLASS
#define INOVA_DLL_IMPORT_FUNCTION HOOPLE_DLL_IMPORT_FUNCTION

// cause the use of either tag to make both tags be defined.
#ifdef USE_INOVA_DLLS 
  #define USE_HOOPLE_DLLS
#elif defined(USE_HOOPLE_DLLS)
  #define USE_INOVA_DLLS
#endif

#endif  // outer guard.

