/*****************************************************************************\
*                                                                             *
*  Name   : chaos                                                             *
*  Author : Chris Koeritz                                                     *
*                                                                             *
*******************************************************************************
* Copyright (c) 1991-$now By Author.  This program is free software; you can  *
* redistribute it and/or modify it under the terms of the GNU General Public  *
* License as published by the Free Software Foundation; either version 2 of   *
* the License or (at your option) any later version.  This is online at:      *
*     http://www.fsf.org/copyleft/gpl.html                                    *
* Please send any updates to: fred@gruntose.com                               *
\*****************************************************************************/

//#define DEBUG_CHAOS
  // uncomment for debugging version.

#include "chaos.h"
#include "log_base.h"
#include "portable.h"
#include "utility.h"

#include <stdlib.h>
#include <time.h>

#ifdef DEBUG_CHAOS
  #define LOG(s) STAMPED_EMERGENCY_LOG(program_wide_logger(), s);
#else
  #define LOG(s) {}
#endif

chaos::chaos() {}

void chaos::retrain()
{
  time_t time_num;
  time(&time_num);
  // "t" is an ugly pointer into system data that contains the time nicely
  // broken up into segments.  the pointer cannot be freed!
  tm *t = localtime(&time_num);
  int add_in_milliseconds = portable::system_uptime();
  // create a good random number seed from the time.
  LOG(isprintf("day %d hour %d min %d sec %d", t->tm_mday, t->tm_hour,
      t->tm_min, t->tm_sec));
  unsigned int temp = (t->tm_sec + 60 * t->tm_mday + 60 * 31 * t->tm_hour
    + 24 * 60 * 31 * t->tm_min) ^ add_in_milliseconds;
  // initialize random generator.
  srand(int(temp));
}

// gets a 32 bit integer from the 15 bit random generator.  it's 15 bit because
// rand() only generates numbers up to the MAX_RAND, which in the visual c++
// case is limited to 0x7FFF.  so, we let the first random number generated
// serve as the upper two bytes and we shift another one over a bit to cover
// the whole range of the third and fourth byte, except for that last bit, 
// which is added in as a binary random value.
#define GET_RAND \
  u_int ranval = (u_int(rand()) << 16) + (u_int(rand()) << 1) \
       + (u_int(rand()) % 2)

int chaos::inclusive(int low, int high)
{
  if (high < low) return low;
  unsigned int range = high - low + 1;
  GET_RAND;
  int adjusted = ranval % range + low;
  return adjusted;
}

int chaos::exclusive(int low, int high)
{
  if (high < low) return low + 1;
  unsigned int range = high - low - 1;
  GET_RAND;
  int adjusted = ranval % range + low + 1;
  return adjusted;
}


