#ifndef CHAOS_CLASS
#define CHAOS_CLASS

/*****************************************************************************\
*                                                                             *
*  Name   : chaos                                                             *
*  Author : Chris Koeritz                                                     *
*                                                                             *
*******************************************************************************
* Copyright (c) 1991-$now By Author.  This program is free software; you can  *
* redistribute it and/or modify it under the terms of the GNU General Public  *
* License as published by the Free Software Foundation; either version 2 of   *
* the License or (at your option) any later version.  This is online at:      *
*     http://www.fsf.org/copyleft/gpl.html                                    *
* Please send any updates to: fred@gruntose.com                               *
\*****************************************************************************/

#include "object_base.h"

//! a platform-independent way to acquire random numbers in a specific range.
/*!
  This object also re-seeds the underlying system's random seed when retrain()
  is invoked.
*/

class chaos : public virtual object_base
{
public:
  chaos();
    //!< constructs a random number generator.  does not retrain random seed.

  IMPLEMENT_CLASS_NAME("chaos");

  void retrain();
    //!< Scrambles the OS's random seed and then provides pseudo-random values.

  // Keep in mind for the inclusive and exclusive functions that a range which
  // is too large might not be dealt with well.  This is because the random
  // functions provided by the O.S. may not support the full range of integers.

  int inclusive(int low, int high);
    //!< Returns a pseudo-random number r, such that "low" <= r <= "high".

  int exclusive(int low, int high);
    //!< Returns a pseudo-random number r, such that "low" < r < "high".
};

#endif

