/*****************************************************************************\
*                                                                             *
*  Name   : utility group                                                     *
*  Authors: Chris Koeritz                                                     *
*                                                                             *
*******************************************************************************
* Copyright (c) 1992-$now By Author.  This program is free software; you can  *
* redistribute it and/or modify it under the terms of the GNU General Public  *
* License as published by the Free Software Foundation; either version 2 of   *
* the License or (at your option) any later version.  This is online at:      *
*     http://www.fsf.org/copyleft/gpl.html                                    *
* Please send any updates to: fred@gruntose.com                               *
\*****************************************************************************/

#include "common_templates.h"
#include "earth_time.h"
#include "istring.h"
#include "portable.h"
#include "utility.h"

#include <stdlib.h>

//#define DEBUG_UTILITY
  // uncomment this macro to cause more debugging checks and verbosity.

const int HIGHEST_MOD_VALUE = 32014;

unsigned int utility::bizarre_checksum(const byte *data, int length)
{
  int sum = 0;
  for (int i = 0; i < length; i++) sum += data[i] % 23 + i % 9;
  sum = (sum % (HIGHEST_MOD_VALUE - 1)) + 1;
  return (unsigned int)sum;
}

// fletcher checksum is from Dr. Dobbs Journal May 1992, pp. 32-38.

uint16 utility::fletcher_checksum(const byte *data, int length)
{
  int sum1 = 0;
  u_int sum2 = 0;
  const byte *buffer = data;

  while (length--) {
    sum1 += int(*buffer++);
    // check for overflow into high byte.
    if (sum1 > 255) sum1++;
    sum1 &= 0xFF;  // remove any bits in high byte for next sum.
    sum2 += u_int(sum1);
  }
  if (sum1 == 255) sum1 = 0;
  unsigned int fletch = u_int(sum2 & 0xFF);
  fletch <<= 8;
  fletch |= u_int(sum1 & 0xFF);

  return uint16(fletch);
}

uint16 utility::rolling_fletcher_checksum(uint16 previous, const byte *data,
    int len)
{ return previous ^ fletcher_checksum(data, len); }

byte utility::byte_checksum(const byte *data, int length)
{
  byte to_return = 0;
  for (int i = 0; i < length; i++) to_return += data[i];
  return to_return;
}

u_int utility::short_checksum(const byte *data, int length)
{
  u_int to_return = 0;
  for (int i = 0; i < length; i++) to_return += data[i];
  return to_return;
}

istring timestamp(bool add_space, bool add_date)
{ return utility::timestamp_string(add_space, add_date); }

istring utility::timestamp_string(bool add_space, bool add_date)
{
  using namespace earth_time;
  const time_locus the_time = now();
  istring to_return;
  if (add_date)
    the_time.text_form(to_return,
        clock_time::MILITARY | clock_time::MILLISECONDS);
  else
    the_time.clock_time::text_form(to_return, clock_time::MILITARY
        | clock_time::MILLISECONDS);
  if (add_space)
    to_return += " ";
  return to_return;
}

