#ifndef INT_HASH_IMPLEMENTATION
#define INT_HASH_IMPLEMENTATION

/*****************************************************************************\
*                                                                             *
*  Name   : int_hash                                                          *
*  Author : Chris Koeritz                                                     *
*                                                                             *
*******************************************************************************
* Copyright (c) 2001-$now By Author.  This program is free software; you can  *
* redistribute it and/or modify it under the terms of the GNU General Public  *
* License as published by the Free Software Foundation; either version 2 of   *
* the License or (at your option) any later version.  This is online at:      *
*     http://www.fsf.org/copyleft/gpl.html                                    *
* Please send any updates to: fred@gruntose.com                               *
\*****************************************************************************/

#include "byte_hasher.h"
#include "hash_table.cpp"
#include "int_hash.h"

#include <basis/common_templates.h>
#include <basis/utility.h>

template <class contents>
int_hash<contents>::int_hash(int max_bits)
: hash_table<int, contents>(rotating_byte_hasher(), max_bits),
  _ids(new int_set)
{}

template <class contents>
int_hash<contents>::~int_hash()
{ WHACK(_ids); }

template <class contents>
const int_set &int_hash<contents>::ids() const { return *_ids; }

template <class contents>
void int_hash<contents>::ids(int_set &ids) const { ids = *_ids; }

template <class contents>
outcome int_hash<contents>::add(int key, contents *to_store)
{
  _ids->add(key);
  return hash_table<int, contents>::add(key, to_store);
}

template <class contents>
contents *int_hash<contents>::acquire(int key)
{
  _ids->remove(key);
  return hash_table<int, contents>::acquire(key);
}

template <class contents>
bool int_hash<contents>::zap(int key)
{
  _ids->remove(key);
  return hash_table<int, contents>::zap(key);
}

template <class contents>
void int_hash<contents>::reset()
{
  _ids->clear();
  hash_table<int, contents>::reset();
}

template <class contents>
void int_hash<contents>::apply(apply_function *to_apply, void *data_link)
{
  for (int i = 0; i < _ids->elements(); i++) {
    int current = (*_ids)[i];
    contents *found = hash_table<int, contents>::find(current);
    if (!found) {
      _ids->remove(current);
      continue;
    }
    to_apply(current, *found, data_link);
  }
}

#endif // outer guard.

