#ifndef STACK_IMPLEMENTATION
#define STACK_IMPLEMENTATION

/*****************************************************************************\
*                                                                             *
*  Name   : stack                                                             *
*  Author : Chris Koeritz                                                     *
*                                                                             *
*******************************************************************************
* Copyright (c) 1990-$now By Author.  This program is free software; you can  *
* redistribute it and/or modify it under the terms of the GNU General Public  *
* License as published by the Free Software Foundation; either version 2 of   *
* the License or (at your option) any later version.  This is online at:      *
*     http://www.fsf.org/copyleft/gpl.html                                    *
* Please send any updates to: fred@gruntose.com                               *
\*****************************************************************************/

#include "stack.h"

#include <basis/array.cpp>
#include <basis/guards.h>

template <class contents>
stack<contents>::stack(int elements)
: array<contents>(elements), _valid_fields(0)
{ _kind = (elements? BOUNDED : UNBOUNDED); }

template <class contents>
stack<contents>::stack(const stack &to_copy)
: array<contents>(0), _valid_fields(0)
{ operator = (to_copy); }

template <class contents> stack<contents>::~stack() {}

template <class contents>
int stack<contents>::size() const { return _valid_fields; }

template <class contents>
int stack<contents>::elements() const { return array<contents>::length(); }

template <class contents>
outcome stack<contents>::push(const contents &element)
{
  if (_kind == BOUNDED) {
    if (_valid_fields >= elements()) return common::IS_FULL;
    outcome result = put(_valid_fields, element);
    if (result != common::OKAY) return common::IS_FULL;
  } else concatenate(element);
  _valid_fields++;
  return common::OKAY;
}

template <class contents>
outcome stack<contents>::pop()
{
  if (_valid_fields < 1) return common::IS_EMPTY;
  if (_kind == UNBOUNDED)
    zap(array<contents>::length() - 1, array<contents>::length() - 1);
  _valid_fields--;
  return common::OKAY;
}

template <class contents>
contents &stack<contents>::top()
{ return array<contents>::operator [](_valid_fields - 1); }

template <class contents>
stack<contents> &stack<contents>::operator = (const stack &to_copy)
{
  if (this == &to_copy) return *this;
  while (size()) pop();
  _kind = to_copy._kind;
  reset(to_copy.length());
  for (int i = 0; i < to_copy.length(); i++)
    put(i, to_copy.array<contents>::get(i));
  _valid_fields = to_copy._valid_fields;
  return *this;
}

template <class contents>
void stack<contents>::invert()
{
  for (int i = 0; i < array<contents>::length() / 2; i++) {
    contents hold = array<contents>::get(i);
    put(i, array<contents>::get(this->length() - i - 1));
    put(this->length() - i - 1, hold);
  }
}

template <class contents>
contents &stack<contents>::operator [](int index)
{
  if (index >= _valid_fields) index = 0;  // force a bogus return.
  return array<contents>::operator [](index);
}

template <class contents>
outcome stack<contents>::acquire_pop(contents &to_stuff)
{
  if (!_valid_fields) return common::IS_EMPTY;
  to_stuff = operator [](_valid_fields - 1);
  if (_kind == UNBOUNDED) zap(elements()-1, elements()-1);
  _valid_fields--;
  return common::OKAY;
}

#endif

