/*****************************************************************************\
*                                                                             *
*  Name   : application_shell                                                 *
*  Author : Chris Koeritz                                                     *
*                                                                             *
*******************************************************************************
* Copyright (c) 2000-$now By Author.  This program is free software; you can  *
* redistribute it and/or modify it under the terms of the GNU General Public  *
* License as published by the Free Software Foundation; either version 2 of   *
* the License or (at your option) any later version.  This is online at:      *
*     http://www.fsf.org/copyleft/gpl.html                                    *
* Please send any updates to: fred@gruntose.com                               *
\*****************************************************************************/

#include "application_shell.h"
#include "console_logger.h"
#include "file_logger.h"
#include "ini_config.h"
#include "path_configuration.h"

#include <basis/chaos.h>
#include <basis/istring.h>
#include <basis/utility.h>

#define INI_NAME(root) (root + istring(".ini"))

application_shell::application_shell(const istring &root, int max_log)
: _rando(new chaos),
  _root_name(new istring(root)),
  _ini(new ini_configurator(INI_NAME(root), ini_configurator::AUTO_STORE))
{}

application_shell::~application_shell()
{
  WHACK(_root_name);
  WHACK(_rando);
  WHACK(_ini);
}

istring application_shell::filename_root() const { return *_root_name; }

ini_configurator &application_shell::ini() const { return *_ini; }

istring application_shell::ini_name() const
{ return INI_NAME(filename_root()); }

istring application_shell::LOG_NAME(const istring &root)
{ return path_configuration::make_logfile_name(root + istring(".log")); }

istring application_shell::log_name() const
{ return LOG_NAME(filename_root()); }

outcome application_shell::log(const istring &to_print)
{ return program_wide_logger().log(to_print); }

application_shell &operator << (application_shell &out, const istring &tp)
{ out.log(tp); return out; }

#ifndef OMIT_PROGRAM_WIDE_LOGGER
log_base &application_shell::logger() const { return program_wide_logger(); }

log_base *application_shell::swap_logger(log_base *new_logger)
{ return retask_program_wide_logger(new_logger); }
#endif

outcome application_shell::log(const istring &to_print, int filter)
{
  if (program_wide_logger().member(filter)) {
    return program_wide_logger().log(to_print);
  }
  return common::OKAY;
}

