#ifndef CONSOLE_LOGGER_CLASS
#define CONSOLE_LOGGER_CLASS

/*****************************************************************************\
*                                                                             *
*  Name   : console_logger                                                    *
*  Author : Chris Koeritz                                                     *
*                                                                             *
*******************************************************************************
* Copyright (c) 2000-$now By Author.  This program is free software; you can  *
* redistribute it and/or modify it under the terms of the GNU General Public  *
* License as published by the Free Software Foundation; either version 2 of   *
* the License or (at your option) any later version.  This is online at:      *
*     http://www.fsf.org/copyleft/gpl.html                                    *
* Please send any updates to: fred@gruntose.com                               *
\*****************************************************************************/

//! A logger that sends to the console screen using the standard output device.

#include "opsysdll.h"

#include <basis/log_base.h>

class OPSYSTEM_CLASS_STYLE console_logger : public log_base
{
public:
  console_logger();
  virtual ~console_logger();

  IMPLEMENT_CLASS_NAME("console_logger");

  virtual outcome log(const istring &info, int filter = ALWAYS_PRINT);
    //!< sends the string "info" to the standard output device.
    /*!< if the "filter" is not in the current filter set, then the
    information will be dropped.  otherwise the information is displayed,
    where appropriate.  for some environments, this will essentially throw
    it away.  for example, in ms-windows, a windowed program will only save
    standard out if one redirects standard output to a file, whereas a
    console mode program will output the text to its parent prompt. */
};

////////////////////////////////////////////////////////////////////////////

#ifndef OMIT_PROGRAM_WIDE_LOGGER
  //! retasks the program wide logger to use a new console_logger.
  /*! the original log_base object is returned and must be dealt
  with appropriately. */
  log_base OPSYSTEM_FUNCTION_STYLE *set_PW_logger_for_console();

  //!< a macro that retasks the program-wide logger as a console_logger.
  #define SET_DEFAULT_CONSOLE_LOGGER { \
    log_base *old_log = set_PW_logger_for_console(); \
    WHACK(old_log); \
  }
#else
  #define SET_DEFAULT_CONSOLE_LOGGER
#endif

#endif

