/*****************************************************************************\
*                                                                             *
*  Name   : critical_events                                                   *
*  Author : Chris Koeritz                                                     *
*                                                                             *
*******************************************************************************
* Copyright (c) 1989-$now By Author.  This program is free software; you can  *
* redistribute it and/or modify it under the terms of the GNU General Public  *
* License as published by the Free Software Foundation; either version 2 of   *
* the License or (at your option) any later version.  This is online at:      *
*     http://www.fsf.org/copyleft/gpl.html                                    *
* Please send any updates to: fred@gruntose.com                               *
\*****************************************************************************/

#include "critical_events.h"
#include "path_configuration.h"

#include <basis/function.h>
#include <basis/istring.h>
#include <basis/log_base.h>
#include <basis/portable.h>
#include <basis/utility.h>
#include <data_struct/static_memory_gremlin.h>

#include <stdio.h>

SAFE_STATIC(istring_object, critical_events::hidden_critical_events_dir, );
  // define the function that holds the directory string.

istring default_critical_location()
{
  // ensure that the critical events function logs to the appropriate place.
  return path_configuration::get_logging_directory();
}

istring critical_events::critical_events_directory()
{
  static bool initted = false;
  if (!initted) {
    auto_synchronizer l(startup_code::user_static_synchronizer());
    if (!initted) {
      set_critical_events_directory(default_critical_location());
      initted = true;
    }
  }
  return hidden_critical_events_dir();
}

void critical_events::set_critical_events_directory(const istring &directory)
{ hidden_critical_events_dir() = directory; }

void critical_events::write_to_critical_events(const char *to_write)
{
  istring filename = critical_events_directory();
  filename += "/runtime_issues.log";
  FILE *errfile = fopen(filename.s(), "ab");
  if (errfile) {
    istring app_name = portable::application_name();
    int indy = app_name.find('/', app_name.end(), true);
    if (non_negative(indy)) app_name.zap(0, indy);
    indy = app_name.find('\\', app_name.end(), true);
    if (non_negative(indy)) app_name.zap(0, indy);
    fprintf(errfile, "%s [%s]:%s", timestamp(true, true).s(),
        app_name.s(), log_base::platform_ending());
    fprintf(errfile, "%s%s", to_write, log_base::platform_ending());
    fclose(errfile);
  }
}

