#ifndef CRITICAL_EVENTS_GROUP
#define CRITICAL_EVENTS_GROUP

/*****************************************************************************\
*                                                                             *
*  Name   : critical_events                                                   *
*  Author : Chris Koeritz                                                     *
*                                                                             *
*******************************************************************************
* Copyright (c) 1989-$now By Author.  This program is free software; you can  *
* redistribute it and/or modify it under the terms of the GNU General Public  *
* License as published by the Free Software Foundation; either version 2 of   *
* the License or (at your option) any later version.  This is online at:      *
*     http://www.fsf.org/copyleft/gpl.html                                    *
* Please send any updates to: fred@gruntose.com                               *
\*****************************************************************************/

//! Provides a means of writing events to a log of runtime problems.

#include "opsysdll.h"

class OPSYSTEM_CLASS_STYLE critical_events
{
public:
  static void write_to_critical_events(const char *message);
    //!< sends the "message" to the critical events log file.
    /*!< Prints out a message to the specially named file that captures all
    serious events written to error logging functions.  If you use the
    functions in this file, you are likely already writing to this log. */

  static void set_critical_events_directory(const istring &directory);
    //!< sets the internal location where the critical events will be logged.
    /*!< this is postponed to a higher level, although the default
    will work too. */

  static istring critical_events_directory();
    //!< returns the current location where critical events are written.

private:
  static istring_object &hidden_critical_events_dir();
};

#endif

