#ifndef FILE_TIME_CLASS
#define FILE_TIME_CLASS

/*****************************************************************************\
*                                                                             *
*  Name   : file_time                                                         *
*  Author : Chris Koeritz                                                     *
*                                                                             *
*******************************************************************************
* Copyright (c) 1992-$now By Author.  This program is free software; you can  *
* redistribute it and/or modify it under the terms of the GNU General Public  *
* License as published by the Free Software Foundation; either version 2 of   *
* the License or (at your option) any later version.  This is online at:      *
*     http://www.fsf.org/copyleft/gpl.html                                    *
* Please send any updates to: fred@gruntose.com                               *
\*****************************************************************************/

//! A platform independent way to obtain the timestamp of a file.

//hmmm: This needs to have some accessing functions provided so that the file
//      time can be printed out and such...

#include "opsysdll.h"

#include <basis/definitions.h>

#include <stdio.h>

#ifdef __BCPLUSPLUS__
  #include <stdlib.h>
  #include <io.h>
  typedef ftime file_time_handle;
#elif defined(_MSC_VER)
  #include <sys/types.h>
  #include <sys/stat.h>
  typedef time_t file_time_handle;
#elif defined(__UNIX__)
  #include <sys/types.h>
  #include <sys/stat.h>
  typedef time_t file_time_handle;
#endif

class OPSYSTEM_CLASS_STYLE file_time
{
public:
  file_time(FILE *the_file = NIL);
    // This constructor sets up the file_time information given a pointer to
    // the stream of interest.  If the stream is NIL, then the file_time is
    // set up as an invalid file_time.

  file_time(const istring &filename);
    // this constructor operates on a filename rather than a FILE stream.

  ~file_time();

  bool valid() const;

  void reset(FILE *the_file);
    // Obtains the file time from a different FILE than it was constructed
    // with, or connects a FILE to the file_time after the empty constructor.
    // this can also be used to refresh a file time when a FILE may have
    // changed.

  bool operator < (const file_time &ft2) const;
  bool operator <= (const file_time &ft2) const;
  bool operator == (const file_time &ft2) const;
  bool operator != (const file_time &ft2) const;
  bool operator > (const file_time &ft2) const;
  bool operator >= (const file_time &ft2) const;
    // Standard comparison operators between this file time and
    // the file time "ft2".  These are meaningless if either time is
    // invalid.

private:
  bool am_i_valid;
  file_time_handle t;

  int compare(const file_time &ft2) const;

  // unimplemented.
  file_time(const file_time &);
  file_time &operator =(const file_time &);
};

#endif

