#ifndef LOCKED_LOGGER_CLASS
#define LOCKED_LOGGER_CLASS

/*****************************************************************************\
*                                                                             *
*  Name   : locked_logger                                                     *
*  Author : Chris Koeritz                                                     *
*                                                                             *
*  Purpose:                                                                   *
*                                                                             *
*    Supports program-wide logging support with safe mechanism swapping by    *
*  implementing an object derived from both locked_object and log_base.       *
*                                                                             *
*******************************************************************************
* Copyright (c) 2002-$now By Author.  This program is free software; you can  *
* redistribute it and/or modify it under the terms of the GNU General Public  *
* License as published by the Free Software Foundation; either version 2 of   *
* the License or (at your option) any later version.  This is online at:      *
*     http://www.fsf.org/copyleft/gpl.html                                    *
* Please send any updates to: fred@gruntose.com                               *
\*****************************************************************************/

#ifndef OMIT_PROGRAM_WIDE_LOGGER

#include "locked_object.h"

#include <basis/log_base.h>

class locked_logger : public log_base, public locked_object<log_base>
{
public:
  locked_logger(log_base *the_logger = NIL);
    // if the stored logger is NIL, then all operations will be ignored.

  virtual ~locked_logger();

  IMPLEMENT_CLASS_NAME("locked_logger");

  bool established();
    // returns true if the log_base has been established with a real
    // pointer.  returns false if the internal log_base is still NIL.

  // override all log_base functionality and re-route it to the real object.
  virtual void add_filter(int new_filter);
  virtual void remove_filter(int old_filter);
  virtual bool member(int filter_to_check);
  virtual void clear_filters();
  virtual int_set filter_set();
  virtual line_ending eol();
  virtual void eol(line_ending to_set);
  virtual istring get_ending();
  virtual void get_ending(istring &to_end);
  virtual outcome log(const istring &info, int filter = ALWAYS_PRINT);

private:
  // not supported.
  locked_logger(const locked_logger &);
  locked_logger &operator =(const locked_logger &);
};

#endif  // OMIT_PROGRAM_WIDE_LOGGER

#endif  // outer guard.

