#ifndef OCCURRENCE_CLASS
#define OCCURRENCE_CLASS

/*****************************************************************************\
*                                                                             *
*  Name   : occurrence                                                        *
*  Author : Chris Koeritz                                                     *
*                                                                             *
*  Purpose:                                                                   *
*                                                                             *
*    Records the occurrence of a particular outcome, the time it happened,    *
*  where it was noted and any additional state information that seemed        *
*  relevant to the recorder.                                                  *
*                                                                             *
*  Note:                                                                      *
*                                                                             *
*    This is a heavyweight header; it should not be used in other headers.    *
*                                                                             *
*******************************************************************************
* Copyright (c) 1990-$now By Author.  This program is free software; you can  *
* redistribute it and/or modify it under the terms of the GNU General Public  *
* License as published by the Free Software Foundation; either version 2 of   *
* the License or (at your option) any later version.  This is online at:      *
*     http://www.fsf.org/copyleft/gpl.html                                    *
* Please send any updates to: fred@gruntose.com                               *
\*****************************************************************************/

#include "opsysdll.h"

#include <basis/earth_time.h>
#include <basis/function.h>
#include <basis/istring.h>
#include <basis/packable.h>
#include <basis/object_base.h>

class OPSYSTEM_CLASS_STYLE occurrence
: public packable, public virtual object_base
{
public:
  outcome _out;  // the particular outcome that was seen.
  earth_time::time_locus _when;  // the time at which the outcome occurred.
  istring _operation;  // the operation that saw fit to record the outcome.
  istring _location;
    // the location where this all happened, plus any other useful state
    // information that might help with debugging or understanding the
    // problem.

  IMPLEMENT_CLASS_NAME("occurrence");

  // the default version of the constructor provides an invalid occurrence.
  inline occurrence(const outcome &o = outcome(1),
      const earth_time::time_locus &when = earth_time::now(),
      const istring &operation = istring::empty_string(),
      const istring &location = istring::empty_string())
      : _out(o), _when(when), _operation(operation), _location(location) {}

  bool valid() const {
    // all valid outcomes are either zero or negative.  they are never valid
    // if their value is a positive integer.
    return non_positive(_out.value());
  }

  virtual void pack(byte_array &packed_form) const;
    // creates a packed form of the occurrence in "packed_form".

  virtual bool unpack(byte_array &packed_form);
    // restores the occurrence from the "packed_form".
};

#endif

