#ifndef REGISTRY_CONFIGURATOR_CLASS
#define REGISTRY_CONFIGURATOR_CLASS

/*****************************************************************************\
*                                                                             *
*  Name   : registry_configurator                                             *
*  Author : Chris Koeritz                                                     *
*                                                                             *
*  Purpose:                                                                   *
*                                                                             *
*    Supports the configurator interface on the windows registry.             *
*                                                                             *
*******************************************************************************
* Copyright (c) 2004-$now By Author.  This program is free software; you can  *
* redistribute it and/or modify it under the terms of the GNU General Public  *
* License as published by the Free Software Foundation; either version 2 of   *
* the License or (at your option) any later version.  This is online at:      *
*     http://www.fsf.org/copyleft/gpl.html                                    *
* Please send any updates to: fred@gruntose.com                               *
\*****************************************************************************/

#include "opsysdll.h"

#include <basis/object_base.h>
#include <data_struct/configurator.h>

// forward.
class byte_filer;
class filename;

class OPSYSTEM_CLASS_STYLE registry_configurator : public configurator
{
public:
  enum registry_hives {
    hkey_classes_root,
    hkey_current_user,
    hkey_local_machine,
    hkey_users,
    hkey_current_config
  };

  registry_configurator(registry_hives hive, treatment_of_defaults behavior);
    // creates a registry_configurator that stores entries into the "hive"
    // and that has the "behavior".

  virtual ~registry_configurator();

  IMPLEMENT_CLASS_NAME("registry_configurator");

  virtual bool get(const istring &section, const istring &entry,
          istring &found);
    // implements the configurator retrieval function.  note that for registry
    // based retrieval, an empty "entry" is allowed, and that specifies the
    // default item in the "section".

  virtual bool section_exists(const istring &section);
    // returns true if the "section" was found in the file.
    // NOTE: for an INI file, this is quite a heavy-weight call.

  virtual bool put(const istring &section, const istring &entry,
          const istring &to_store);
    // implements the configurator storage function.  put interprets an empty
    // string for "entry" as pointing at the default item in the "section".

  virtual bool delete_section(const istring &section);
    // removes the entire "section" specified.

  virtual bool delete_entry(const istring &section, const istring &entry);
    // removes the entry specified by the "section" and "entry" name.

  virtual bool get_section(const istring &section, string_table &info);
    // reads the entire "section" into a table called "info".  on win95, this
    // will fail if the section's data exceeds 32K.

  virtual bool put_section(const istring &section, const string_table &info);
    // writes a table called "info" into the "section" in the INI file.  any
    // existing data for that section is wiped out.  on win95, this will fail
    // if the section's data exceeds 32K.

  void *translate_hive(registry_hives hive);
    // translates from our enum to the windows specific type for hives.

  istring fix_section(const istring &section);
    // translates a section name that might have forward slashes into the
    // mandatory backslashes for windows.

private:
  registry_hives _hive;  // which hive these entries are stored in.

  // not to be called.
  registry_configurator(const registry_configurator &);
  registry_configurator &operator =(const registry_configurator &);

  static const istring_object &reg_str_fake_default();
};

#endif

