#ifndef ITEM_STATISTICS_CLASS
#define ITEM_STATISTICS_CLASS

/*****************************************************************************\
*                                                                             *
*  Name   : item_statistics                                                   *
*  Author : Chris Koeritz                                                     *
*                                                                             *
*  Purpose:                                                                   *
*                                                                             *
*    This is the set of timing information collected for a scheduled item.    *
*  Note that this header is intended for use in CPP files and should be       *
*  forward declared within other headers.                                     *
*                                                                             *
*******************************************************************************
* Copyright (c) 1998-$now By Author.  This program is free software; you can  *
* redistribute it and/or modify it under the terms of the GNU General Public  *
* License as published by the Free Software Foundation; either version 2 of   *
* the License or (at your option) any later version.  This is online at:      *
*     http://www.fsf.org/copyleft/gpl.html                                    *
* Please send any updates to: fred@gruntose.com                               *
\*****************************************************************************/

#include "scheduling_dll.h"

#include <data_struct/guarded_value.h>
#include <mathematics/common_templates.h>
#include <mechanisms/time_stamp.h>

class SCHEDULING_CLASS_STYLE item_statistics
{
public:
  guarded_value<time_stamp> entry;  // when the item was preprocessed for entry.
  guarded_value<time_stamp> exit;  // when the item left the schedule.
  guarded_value<int> last_outcome;  // what happened when we last scheduled.
  int_averager durations;  // record of activation times.

  inline item_statistics(int durations_kept = 14)
          : entry(), exit(), last_outcome(), durations(durations_kept) {}
    // sets up the statistics package.  creates our averager and sets the
    // timers to invalid values.  the proper type for "last_outcome" is
    // unknown, so we leave it uninitialized too.

  istring text_form() const;
    // returns a string with a text representation of our information.

  istring adjusted_representation
          (const guarded_value<time_stamp> &to_show) const;
    // prints out either the time stamp's value or "invalid" if it has no
    // value yet.
};

#endif

