#ifndef ENCRYPTION_INFOTON_CLASS
#define ENCRYPTION_INFOTON_CLASS

/*****************************************************************************\
*                                                                             *
*  Name   : encryption_infoton                                                *
*  Author : Chris Koeritz                                                     *
*                                                                             *
*  Purpose:                                                                   *
*                                                                             *
*    Encapsulates the chit-chat necessary to establish an encrypted           *
*  connection.  This is framed in terms of a client and a server, where the   *
*  client creates a private key and gives the server the public key.  the     *
*  server side creates a blowfish key and encrypts it using the public key.   *
*                                                                             *
*******************************************************************************
* Copyright (c) 2004-$now By Author.  This program is free software; you can  *
* redistribute it and/or modify it under the terms of the GNU General Public  *
* License as published by the Free Software Foundation; either version 2 of   *
* the License or (at your option) any later version.  This is online at:      *
*     http://www.fsf.org/copyleft/gpl.html                                    *
* Please send any updates to: fred@gruntose.com                               *
\*****************************************************************************/

#ifndef OMIT_CRYPTO_SUPPORT

#include "dll_tentacles.h"

#include <octopus/entity_defs.h>
#include <octopus/infoton.h>

// forward.
class blowfish_crypto;
class RSA_crypto;

class TENTACLES_CLASS_STYLE encryption_infoton : public infoton
{
public:
  byte_array _public_key;
    // valid during the request stage of encryption, when the client is
    // telling the server how to talk to it to provide the key.
  byte_array _encrypted_blowfish_key;
    // valid during the response stage of encryption, when the server reports
    // a blowfish key that it will use on this connection.

  outcome _success;  // did the request succeed?

  encryption_infoton(const byte_array &public_key = byte_array::empty_array(),
          const byte_array &encrypted_blowfish_key = byte_array::empty_array());
  encryption_infoton(const encryption_infoton &to_copy);

  virtual ~encryption_infoton();

  IMPLEMENT_CLASS_NAME("encryption_infoton");

  static const int RSA_KEY_SIZE;
    // this key size should be used for all RSA private keys.
  static const int BLOWFISH_KEY_SIZE;
    // this will be used for blowfish keys that this object generates.

  encryption_infoton &operator =(const encryption_infoton &to_copy);

  outcome prepare_blowfish_key(blowfish_crypto &new_key);
    // does the server's job on the current key.  the public key had better
    // be set already or this will fail.  the "new_key" will always be used to
    // communicate with the client after this.

  outcome prepare_public_key(const RSA_crypto &private_key);
    // prepares the request side for a client.  the rsa public key will be
    // generated from the "private_key".

  outcome prepare_both_keys(RSA_crypto &private_key);
    // sets up both keys by randomly generating the "private_key".

  outcome extract_response(const RSA_crypto &private_key,
          blowfish_crypto &new_key) const;
    // using the private key, the server's response is decrypted and stored
    // in "new_key".  note that this will only succeed if the _success member
    // is OKAY.  otherwise it means the server has beefed on the request.

  static const string_array &encryption_classifier();
    // returns the classifier for this type of infoton.

  virtual void pack(byte_array &packed_form) const;
  virtual bool unpack(byte_array &packed_form);

  virtual clonable *clone() const { return cloner<encryption_infoton>(*this); }

  virtual int packed_size() const;
};

#endif  // crypto support.

#endif  // outer guard.

