#ifndef ENCRYPTION_WRAPPER_CLASS
#define ENCRYPTION_WRAPPER_CLASS

/*****************************************************************************\
*                                                                             *
*  Name   : encryption_wrapper                                                *
*  Author : Chris Koeritz                                                     *
*                                                                             *
*  Purpose:                                                                   *
*                                                                             *
*    Wraps an encrypted infoton when the communication with the octopus is    *
*  encrypted.  The package will be unwrapped by the encryption tentacle.      *
*                                                                             *
*******************************************************************************
* Copyright (c) 2004-$now By Author.  This program is free software; you can  *
* redistribute it and/or modify it under the terms of the GNU General Public  *
* License as published by the Free Software Foundation; either version 2 of   *
* the License or (at your option) any later version.  This is online at:      *
*     http://www.fsf.org/copyleft/gpl.html                                    *
* Please send any updates to: fred@gruntose.com                               *
\*****************************************************************************/

#ifndef OMIT_CRYPTO_SUPPORT

#include "dll_tentacles.h"

#include <octopus/entity_defs.h>
#include <octopus/infoton.h>
#include <octopus/tentacle_helper.h>

class TENTACLES_CLASS_STYLE encryption_wrapper : public infoton
{
public:
  byte_array _wrapped;
    // the encrypted data that's held here.  this must be a packed classifier
    // string array followed by the packed infoton.

  encryption_wrapper(const byte_array &wrapped = byte_array::empty_array());

  encryption_wrapper(const encryption_wrapper &to_copy);

  virtual ~encryption_wrapper();

  IMPLEMENT_CLASS_NAME("encryption_wrapper");

  encryption_wrapper &operator =(const encryption_wrapper &to_copy);

  static const string_array &encryption_classifier();
    // returns the classifier for this type of infoton.

  virtual void pack(byte_array &packed_form) const;
  virtual bool unpack(byte_array &packed_form);

  virtual clonable *clone() const { return cloner<encryption_wrapper>(*this); }

  virtual int packed_size() const;
};

////////////////////////////////////////////////////////////////////////////

// this simple tentacle just unpacks the encryption_wrapper infoton.  it
// never should be doing more than that.

class TENTACLES_CLASS_STYLE unwrapping_tentacle
: public tentacle_helper<encryption_wrapper>
{
public:
  unwrapping_tentacle();
  virtual ~unwrapping_tentacle();

  IMPLEMENT_CLASS_NAME("unwrapping_tentacle");

  virtual outcome reconstitute(const string_array &classifier,
          byte_array &packed_form, infoton * &reformed);

  virtual outcome consume(infoton &to_chow, const octopus_request_id &item_id,
          byte_array &transformed);
    // should never be called.
};

#endif  // crypto support.

#endif  // outer guard.

