/*****************************************************************************\
*                                                                             *
*  Name   : key_repository                                                    *
*  Author : Chris Koeritz                                                     *
*                                                                             *
*******************************************************************************
* Copyright (c) 2004-$now By Author.  This program is free software; you can  *
* redistribute it and/or modify it under the terms of the GNU General Public  *
* License as published by the Free Software Foundation; either version 2 of   *
* the License or (at your option) any later version.  This is online at:      *
*     http://www.fsf.org/copyleft/gpl.html                                    *
* Please send any updates to: fred@gruntose.com                               *
\*****************************************************************************/

#ifndef OMIT_CRYPTO_SUPPORT

#include "key_repository.h"

#include <basis/auto_synch.h>
#include <basis/utility.h>
#include <crypto/blowfish_crypto.h>
#include <data_struct/symbol_table.cpp>

#undef LOG
#define LOG(s) CLASS_EMERGENCY_LOG(program_wide_logger(), s)

//#define DEBUG_KEY_REPOSITORY
  // uncomment for noisier execution.  beware however, if the uls is in
  // use, this can cause infinite recursion.

octenc_key_record *key_repository::lock(const octopus_entity &ent)
{
  #define func "lock"
#ifdef DEBUG_KEY_REPOSITORY
  LOG(istring("entity sought=") + ent.text_form());
#endif
  octenc_key_record *to_return = NIL;
  _locker.lock();
  to_return = _keys.find(ent.mangled_form());
  if (!to_return) {
#ifdef DEBUG_KEY_REPOSITORY
    LOG(istring("did not find entity=") + ent.text_form());
#endif
    _locker.unlock();
  } else {
#ifdef DEBUG_KEY_REPOSITORY
    LOG(istring("found entity=") + ent.text_form());
#endif
  }
  return to_return;
  #undef func
}

void key_repository::unlock(octenc_key_record *to_unlock)
{
  if (!to_unlock) return;  // dolts!  they cannot unlock a non-record.
  _locker.unlock();
}

outcome key_repository::add(const octopus_entity &ent,
    const blowfish_crypto &key)
{
  #define func "add"
  auto_synchronizer loc(_locker);
#ifdef DEBUG_KEY_REPOSITORY
  LOG(istring("adding key for entity=") + ent.text_form());
#endif
  octenc_key_record rec(ent, key);
  return _keys.add(ent.mangled_form(), rec);
  #undef func
}

outcome key_repository::whack(const octopus_entity &ent)
{
  #define func "whack"
  auto_synchronizer loc(_locker);
#ifdef DEBUG_KEY_REPOSITORY
  LOG(istring("removing key for entity=") + ent.text_form());
#endif
  return _keys.whack(ent.mangled_form());
  #undef func
}

#endif // crypto

