#ifndef COLUMN_HEADERS_CLASS
#define COLUMN_HEADERS_CLASS

/*****************************************************************************\
*                                                                             *
*  Name   : column_headers                                                    *
*  Author : Chris Koeritz                                                     *
*                                                                             *
*******************************************************************************
* Copyright (c) 2000-$now By Author.  This program is free software; you can  *
* redistribute it and/or modify it under the terms of the GNU General Public  *
* License as published by the Free Software Foundation; either version 2 of   *
* the License or (at your option) any later version.  This is online at:      *
*     http://www.fsf.org/copyleft/gpl.html                                    *
* Please send any updates to: fred@gruntose.com                               *
\*****************************************************************************/

#include "textual_dll.h"

#include <basis/packable.h>

// forward.
class column_info_array;
class string_matrix;

//! Provides a simple way to design a table structure.
/*!
  The column headers are managed here and allow a specification of maximum
  width, justification, and other properties of the column.
*/

class TEXTUAL_CLASS_STYLE column_headers : public packable
{
public:
  column_headers();
  virtual ~column_headers();

  int columns() const;
    //!< returns the current number of columns in the header list.

  enum justifications { LEFT_JUSTIFY, CENTER_JUSTIFY, RIGHT_JUSTIFY };

  void add(const istring &column_name, justifications placement = LEFT_JUSTIFY,
          int maximum_width = 0);
    //!< adds a new column at the end of the list of columns with "column_name".
    /*!< it's text justification will be "placement".  if "maximum_width" is
    non-zero, then text printed to the column can only be that many
    characters or less. */

  bool get(int index, istring &column_name, justifications &placement,
          int &maximum_width);
    //!< returns true if the column's info can be stuffed into the parameters.

  bool zap(int start, int end = -1);
    //!< destroys the columns between "start" and "end".
    /*!< if "end" is negative, then only the column at "start" is zapped.
    true is returned on success. */

  bool format(istring &to_fill, const string_array &to_format);
    //!< applies our formatting to the array "to_format".
    /*!< returns true if the formatting of this set of column headers can be
    applied to the list of strings in "to_format".  if successful, "to_fill"
    gets stuffed with a formatted line of text based on the column design.
    the method can fail if there are too many or too few headers for the
    list in "to_format".  the "to_fill" string is appended to rather than
    overwritten. */

  bool format(istring &to_fill, const string_matrix &to_format);
    //!< similar to the format() above, but operates on a matrix of strings.
    /*!< this allows the output in "to_fill" to be tailored to the
    available column widths; any column that was defined without a maximum
    width will be expanded as necessary.  if any column that does have a
    maximum width's entries are smaller than the maximum, then the column
    width is shrunk appropriately. */

  virtual void pack(byte_array &packed_form) const;
    //!< packs the column headers into the byte_array "packed_form".
  virtual bool unpack(byte_array &packed_form);
    //!< unpacks the column headers from the byte_array "packed_form".

private:
  column_info_array *_cols;  //!< our representation of the column headers.
};

#endif

